import socket
import json
import time
import os
from netq_lib.common.nlfullstate import NetlinkFullStateObject, FullState
from netq_lib.orm.redisdb.models import (
    Address,
    Command,
    Description,
    Link,
    MacFdb,
    Neighbor,
    Node,
    Route,
    Services,
    VxlanRemoteDest,
    Netconf
)


class AgentCli(object):

    __MAX_LENGTH = 4096

    __SERVER = '/run/netq_cli_server'

    def __init__(self, pool, logger, nlmonitor):
        self.__commands = [""]
        self.__supported_models = []
        self.__pool = pool
        self._logger = logger
        self.__nlmonitor = nlmonitor
        self.full_state_object = NetlinkFullStateObject
        self.__object_names = ["Address", "Command", "Description", "Link",
                               "MacFdb", "Neighbor", "Node", "Route",
                               "Services", "VxlanRemoteDest", "Netconf"]


    def _write_to_file(self, buf, objs):

        #now = datetime.now()
        #file_name = buf + "_" + now.strftime("%m_%d_%Y_%H_%M_%S")
        abs_path = "/var/log/netq-model-" + buf
        try:
            out_file = open(abs_path, "w+")
        except Exception as e:
            self._logger.error("Unable to write to dump file {}".format(e))
            return

        out_file.write("[")
        for obj in objs:
            outstr = json.dumps(obj.to_json())
            out_file.write(outstr)
            out_file.write(",\n")
        out_file.write("]")

        out_file.close()
        return

    @FullState("Route")
    def __netlink_get_all_routes(self):
        """ Get netlink neigh
        """
        self.__pool.spawn(self.__nlmonitor.get_all_routes).wait()
        self.__nlmonitor.set_mcast_group('route')

    def get_nlmonitor(self):
        return self.__nlmonitor


    def _dump_all_objects(self):

        for obj_name in self.__object_names:
            model_objs = self.__get_mem_objects(obj_name)
            if model_objs:
                    self._write_to_file(obj_name, model_objs)

    def __process_cli_command(self, clientsocket):
        try:
            while True:
                buf = clientsocket.recv(self.__MAX_LENGTH)
                if buf == '':
                    return #client terminated connection
                self._logger.info("Received Command to dump {}".format(buf))
                if buf == "all":
                    self._dump_all_objects()
                else:
                    all_objs = self.__get_mem_objects(buf)
                    if all_objs:
                        self._write_to_file(buf, all_objs)
        finally:
            clientsocket.close()

    def __get_mem_objects(self, objectname, generator=True):
        try:
            if(objectname == "Route" or objectname == "Routes"):
                mcls = Route.memmodel()
            elif(objectname == "Command"):
                mcls = Command.memmodel()
            elif(objectname == "Description"):
                mcls = Description.memmodel()
            elif(objectname == "Link"):
                mcls = Link.memmodel()
            elif(objectname == "MacFdb"):
                mcls = MacFdb.memmodel()
            elif(objectname == "Neighbor"):
                mcls = Neighbor.memmodel()
            elif(objectname == "Node"):
                mcls = Node.memmodel()
            elif(objectname == "Services"):
                mcls = Services.memmodel()
            elif(objectname == "VxlanRemoteDest"):
                mcls = VxlanRemoteDest.memmodel()
            elif(objectname == "Netconf"):
                mcls = Netconf.memmodel()
            elif(objectname == "Address"):
                mcls = Address.memmodel()
            else:
                mcls = eval(objectname).memmodel()
        except Exception:
            self._logger.error("Unable to find obj {}".format(objectname))
            return None
        if generator:
            return mcls.query.all()
        else:
            all_objs = [obj for obj in mcls.query.all()]
        return all_objs


    def __compare_route(self, old_obj, new_obj):

        if ((old_obj.vrf == new_obj.vrf) and (old_obj.prefix == new_obj.prefix)):
            return True
        else:
            return False

    def __compare_routes(self, old_objs, new_objs):

        old_stale = []

        if ((len(old_objs) == 0) or (len(new_objs) == 0)):
            self._logger.info("Ignoring initial runs")
            return

        valid_routes = new_objs
        inner_compare = False

        for old_obj in old_objs:
            inner_compare = False
            for idx, new_obj in enumerate(valid_routes):
                cmp_res = self.__compare_route(old_obj, new_obj)
                if cmp_res:
                    inner_compare = True
                    del valid_routes[idx]
                    break
            if inner_compare is False:
                self._logger.info("Old Object no match prefix {} vrf {}".format(old_obj.prefix, old_obj.vrf))
                old_stale.append(old_obj)

        for route in valid_routes:
            self._logger.info("New Object no match prefix {} vrf {}".format(old_obj.prefix, old_obj.vrf))

        
        if ((len(old_stale) > 0) or (len(valid_routes) > 0)):
            self._logger.info("Error found \n")
            self._write_to_file("old_stale", old_stale)   
            self._write_to_file("new_stale", valid_routes)
            return
        else:
            self._logger.info("Old objects matches recent ones")

    def __process_periodic_runner(self):
        self.__RUN_PERIOD = 30
        object_name = "Route"

        while True:

            time.sleep(5)

            old_objs = self.__get_mem_objects(object_name, False)
            if old_objs is None:
                continue

            if (object_name == "Route"):
                self.__netlink_get_all_routes()

                new_objs = self.__get_mem_objects(object_name, False)

                self.__compare_routes(old_objs, new_objs)


    def process_cli_server(self):

        # Make sure the socket does not already exist
        try:
            os.unlink(self.__SERVER)
        except OSError:
            if os.path.exists(self.__SERVER):
                raise
        
        serversocket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

        serversocket.bind(self.__SERVER)
        serversocket.listen(10)
        
        #debug_thread = self.__pool.spawn(self.__process_periodic_runner) 

        while True:
            (clientsocket, address) = serversocket.accept()
            self.__pool.spawn(self.__process_cli_command, clientsocket)
            


